package com.hai.pay.channel.huift;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.HttpsUtils;
import com.hai.config.CommonSysConst;
import com.hai.entity.HighOrder;
import com.hai.entity.HighOrderRefund;
import com.hai.pay.channel.huift.config.HuiftConfig;
import com.hai.service.HighOrderService;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * @className: TradeService
 * @author: HuRui
 * @date: 2024/4/7
 **/
@Component
public class TradeService {

    @Resource
    private HighOrderService orderService;

    /**
     * 交易-预下单
     * @param order
     * @return
     * @throws Exception
     */
    public JSONObject thirdPreOrder(HighOrder order) throws Exception {
        Map<String,Object> param = new HashMap<>();
        param.put("shopCode", HuiftConfig.shopcode);
        param.put("outTradeNo", order.getOrderNo());
        param.put("transAmt", order.getPayablePrice().multiply(new BigDecimal("100")).intValue());
        param.put("mobileNbr", order.getMemPhone()); // 用户标识
        param.put("callbackUrl", CommonSysConst.getSysConfig().getHsgDomainName() + "/order/payNotify/huiftPay");
        // param.put("remark", ""); // 备注
        // param.put("outAppId", "");
        // param.put("outRedirectPath", "");
        param.put("sign", HuiftConfig.md5Encode(HuiftConfig.paramSort(param, HuiftConfig.signSecret).getBytes()).toLowerCase());
        JSONObject response = HttpsUtils.doPost(HuiftConfig.reqUrl + "/hfq/v1/thirdPay/thirdPreOrder", JSONObject.toJSONString(param));
        if (response.getString("status").equals("ok")) {
            return response.getJSONObject("data");
        }
        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR , response.getString("message"));
    }

    /**
     * 查询交易结果
     * @param preOrderNo
     * @return
     * @throws Exception
     */
    public JSONObject thirdPayResult(String preOrderNo) throws Exception {
        Map<String,Object> param = new HashMap<>();
        param.put("shopCode", HuiftConfig.shopcode);
        param.put("preOrderNo", preOrderNo);
        param.put("sign", HuiftConfig.md5Encode(HuiftConfig.paramSort(param, HuiftConfig.signSecret).getBytes()).toLowerCase());
        JSONObject response = HttpsUtils.doPost(HuiftConfig.reqUrl + "/hfq/v1/thirdPay/thirdPayResult", JSONObject.toJSONString(param));
        if (response.getString("status").equals("ok")) {
            return response.getJSONObject("data");
        }
        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR , response.getString("message"));
    }

    /**
     * 交易退款
     * @param orderRefund
     * @return
     * @throws Exception
     */
    public JSONObject thirdPayRefund(HighOrderRefund orderRefund) throws Exception {
        // 查询订单
        HighOrder order = orderService.getOrderById(orderRefund.getId());

        Map<String,Object> param = new HashMap<>();
        param.put("shopCode", HuiftConfig.shopcode);
        param.put("preOrderNo", order.getPayChannelOrderNo());
        param.put("outRefundNo", orderRefund.getRefundOrderNo());
        param.put("refundAmt", orderRefund.getRefundPrice().multiply(new BigDecimal("100")).intValue());
        param.put("sign", HuiftConfig.md5Encode(HuiftConfig.paramSort(param, HuiftConfig.signSecret).getBytes()).toLowerCase());
        JSONObject response = HttpsUtils.doPost(HuiftConfig.reqUrl + "/hfq/v1/thirdPay/thirdPayRefund", JSONObject.toJSONString(param));
        if (response.getString("status").equals("ok")) {
            return response.getJSONObject("data");
        }
        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR , response.getString("message"));
    }

}
